/******************************************************************************/
/* Files to Include                                                           */
/******************************************************************************/

#include <htc.h>            /* HiTech General Includes */
#include <stdint.h>         /* For uint8_t definition */
#include <stdbool.h>        /* For true/false definition */

#include "user.h"
#include "i2c.h"
#include "OLIMEX.h"

/******************************************************************************/
/* User Functions                                                             */
/******************************************************************************/

/* <Initialize variables in user.h and insert code for user algorithms.> */



void InitApp(void)
{
	/* Setup analog functionality and port direction */

	ANSELA = 0;
	ANSELC = 0;

	JMP_TRIS = 1;
	JMP_LAT = 0;

	SDA_TRIS = 1;
	SCL_TRIS = 1;
	SDA_LAT = 0;
	SCL_LAT = 0;


	DE_TRIS = 0;
	nRE_TRIS = 0;
	R_TRIS = 1;
	D_TRIS = 0;

	TXD_TRIS = 1;
	RXD_TRIS = 1;






	/* Initialize peripherals */

	/*CLC Setup*/
	APFCONbits.CLC1SEL = 1;
	NCO1CLKbits.N1CKS = 11;
	NCO1CONbits.N1EN = 1;


	// File: CLC2.inc
	// Generated by CLC Designer, Version: 1.0.0.3
	// Date: 21/04/2012 11:05
	// Device:PIC16(L)F1503

	CLC1GLS0 = 0x02;
	CLC1GLS1 = 0x00;
	CLC1GLS2 = 0x00;
	CLC1GLS3 = 0x00;
	CLC1SEL0 = 0x01;
	CLC1SEL1 = 0x00;
	CLC1POL = 0x02;
	CLC1CON = 0x00;

	CLC2GLS0 = 0x02;
	CLC2GLS1 = 0x00;
	CLC2GLS2 = 0x00;
	CLC2GLS3 = 0x00;
	CLC2SEL0 = 0x01;
	CLC2SEL1 = 0x00;
	CLC2POL = 0x02;
	CLC2CON = 0x00;

	/* Enable interrupts */
}

void InitAddress(void)
{
	unsigned char address;
	address = ReadFlash(ADDR_MEM);
	/* IF THE MEMORY CELL IS EMPTY*/
	if ((address & 0x00FF) == 0xFF)
		ADDRESS = ADDR_DFL;
	else
		ADDRESS = address;
}

unsigned int ReadFlash(unsigned int address)
{
	unsigned int data = 0;
	PMADRL = address & 0x00FF;
	address >>= 8;
	PMADRH = address & 0x00FF;
	PMCON1bits.CFGS = 0;
	PMCON1bits.RD = 1;
	Nop();
	Nop();

	data = PMDATH;
	data <<= 8;
	data |= PMDATL;
	return data;
}

void EraseFlash(unsigned int address)
{
	GIE = 0;
	PMADRL = address & 0x00FF;
	address >>= 8;
	PMADRH = address & 0x00FF;
	PMCON1bits.CFGS = 0;
	PMCON1bits.FREE = 1;
	PMCON1bits.WREN = 1;
	UnlockFlash();
	PMCON1bits.WREN = 0;
	GIE = 1;
}

void UnlockFlash(void)
{
	PMCON2 = 0x55;
	PMCON2 = 0xAA;
	PMCON1bits.WR = 1;
	Nop();
	Nop();
}

void WriteFlash(unsigned int data, unsigned int address)
{
	GIE = 0;
	PMCON1bits.CFGS = 0;
	PMADRL = address & 0x00FF;
	address >>= 8;
	PMADRH = address & 0x00FF;
	PMCON1bits.FREE = 0;
	PMCON1bits.LWLO = 1;
	PMCON1bits.WREN = 1;
	PMDATL = data & 0x00FF;
	data >>= 8;
	PMDATH = data & 0x00FF;
	PMCON1bits.LWLO = 0;
	UnlockFlash();
	PMCON1bits.WREN = 0;
	GIE = 1;
}

void CommandSetCLC(unsigned char data)
{
	unsigned char read, write;

	write = (data & 0x01) ? 1 : 0;
	read = (data & 0x02) ? 1 : 0;

	// Configure write
	DE_LAT = write;
	CLC1CONbits.LC1EN = write;
	CLC1CONbits.LC1OE = write;

	//Configure read
	RXD_TRIS = !read;
	nRE_LAT = !read;
	CLC2CONbits.LC2EN = read;
	CLC2CONbits.LC2OE = read;
}

void CommandSetAddress(unsigned char data)
{
	if (!JMP_PORT) {
		/* STORE THE NEW ADDRESS INTO THE FLASH MEMORY */
		EraseFlash(ADDR_MEM);
		WriteFlash(data, ADDR_MEM);
		ADDRESS = data;
	}
}

void StartSystem(void)
{
	unsigned char address = 0;
	unsigned char command = 0;
	unsigned char data = 0;
	unsigned char rw = 0;
	unsigned char temp_data = 0;
	unsigned char i;
	unsigned char j = 0;

	/*Loop until START condition is recieved*/
	StartI2C();

	/*Read the first bit of the address*/
	address = ReadByteI2C();
	SCL_TRIS = 0; //Hold the clock until the end of the check
	if ((address >> 1) == ADDRESS) {
		/* Send ACK */
		SDA_TRIS = 0;
		SCL_TRIS = 1;
		while (!SCL_PORT);
		while (SCL_PORT);
		SDA_TRIS = 1;

		SCL_TRIS = 0; //HOLD THE CLOCK
		rw = address & 0x01;
		if (rw) {
			while (1) {
				temp_data = DATA_OUT[j++];
				/* READ LAST WAS ADDRESS */
				SDA_TRIS = (temp_data & 0x80) ? 1 : 0;
				temp_data <<= 1;
				SCL_TRIS = 1; //RELEASE THE CLOCK
				for (i = 0; i < 7; i++) {
					while (!SCL_PORT);
					while (SCL_PORT);
					SDA_TRIS = (temp_data & 0x80) ? 1 : 0;
					temp_data <<= 1;
				}


				while (!SCL_PORT);
				while (SCL_PORT);
				SDA_TRIS = 1;
				while (!SCL_PORT);
				if (SDA_PORT)
					break;
				while (SCL_PORT);
			}
			DATA_OUT[0] = 0x00;
			DATA_OUT[1] = 0x00;
			DATA_OUT[2] = 0x00;
			DATA_OUT[3] = 0x00;
		} else {
			/* WRITE LAST WAS ADDRESS */
			SCL_TRIS = 1;
			command = ReadByteI2C();
			SendAck();
			SCL_TRIS = 0;
			switch (command) {
			case SET_CLC:
				SCL_TRIS = 1;
				data = ReadByteI2C();
				SendAck();
				SCL_TRIS = 0;
				CommandSetCLC(data);
				SCL_TRIS = 1;
				break;
			case SET_ADDRESS:
				SCL_TRIS = 1;
				data = ReadByteI2C();
				SendAck();
				SCL_TRIS = 0;
				CommandSetAddress(data);
				SCL_TRIS = 1;
				break;
			case GET_ID:
				DATA_OUT[0] = DEVICE_ID;
				SCL_TRIS = 1;
				break;
			default:
				SCL_TRIS = 1;
				break;
			}
		}



	} else {
		SCL_TRIS = 1;
		SendNack();
	}

	StopI2C();



}

